<?php

/**
 * @file
 * Contains Drupal\tvi\Controller\TaxonomyViewsIntegratorTermPageController.
 */

namespace Drupal\tvi\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\taxonomy\TermInterface;
use Drupal\views\Views;
use Drupal\Core\Config;

class TaxonomyViewsIntegratorTermPageController extends ControllerBase {

  /**
   * Returns a renderable array from a View based on the taxonomy view integrator configuration settings for the requested term entity.
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   * @return \Symfony\Component\HttpFoundation\Response
   */
  public function render(TermInterface $taxonomy_term) {
    return $this->getTaxonomyTermView($taxonomy_term);
  }

  /**
   * Get the taxonomy view integrator settings for this term entity.
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   * @return \Drupal\Core\Config\Config
   */
  protected function getTermConfigSettings(TermInterface $taxonomy_term) {
    return $this->config('tvi.taxonomy_term.' . $taxonomy_term->id());
  }

  /**
   * Get the taxonomy view integrator settings for this terms vocabulary entity.
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   * @return \Drupal\Core\Config\Config
   */
  protected function getVocabularyConfigSettings(TermInterface $taxonomy_term) {
    return $this->config('tvi.taxonomy_vocabulary.' . $taxonomy_term->getVocabularyId());
  }

  /**
   * Get the default taxonomy view integrator settings.
   * @return \Drupal\Core\Config\Config
   */
  protected function getDefaultConfigSettings() {
    return $this->config('tvi.global_settings');
  }

  /**
   * Return the taxonomy term View per taxonomy view integrator settings.
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   * @return bool|null|string|void
   */
  protected function getTaxonomyTermView(TermInterface $taxonomy_term) {
    $config = $this->getTermConfigSettings($taxonomy_term);
    $check_vocab = true;
    $check_global = true;
    $current_uri = \Drupal::request()->getRequestUri();

    // if we have no matches, we are going to call the default term view that ships with core Drupal
    $view_name = 'taxonomy_term';
    $view_id = 'page_1';
    $view_arguments = [$taxonomy_term->id()];

    // View return scenarios:
    //   term is TVI configured, most specific override
    //   term is a child of another term, its top most TVI enabled parent has 'children inherit' option enabled
    //   no term is enabled, but the vocabulary is tvi enabled
    //   no term tvi, no vocab tvi, but global tvi settings exist and enabled
    //   nothing enabled, return default taxonomy term view

    if ($config->get('enable_override')) {
      $view_name = $config->get('view');
      $view_id = $config->get('view_display');
    } else {
      // @todo the logic in this block definitely could use refactoring

      // check parent terms for settings
      // if parent is enabled and has 'children inherit settings' is checked off
      foreach ($this->getTermParents($taxonomy_term) as $parent) {
        // skip the current term, it is returned by loadAllParents
        if ($taxonomy_term->id() === $parent->id()) {
          continue;
        }

        $config = $this->getTermConfigSettings($parent);

        if ($config->get('enable_override') && $config->get('inherit_settings')) {
          $view_name = $config->get('view');
          $view_id = $config->get('view_display');
          $check_vocab = false;
          break;
        }
      }

      // check vocab for settings
      if ($check_vocab) {
        $config = $this->getVocabularyConfigSettings($taxonomy_term);

        if ($config->get('enable_override') && $config->get('inherit_settings')) {
          $view_name = $config->get('view');
          $view_id = $config->get('view_display');
          $check_global = false;
        }
      }

      // check for global settings
      if ($check_global) {
        $config = $this->getDefaultConfigSettings();

        if ($config->get('enable_override')) {
          $view_name = $config->get('view');
          $view_id = $config->get('view_display');
        }
      }
    }

    // if the option to pass all args to views is enabled, pass them on
    if ($config->get('pass_arguments')) {
      $view_arguments += array_slice(explode('/', $current_uri), 3);
    }

    // @todo: page views can override the term page title set in the view, but how can we do that for a block display?
    return Views::getView($view_name)->executeDisplay($view_id, $view_arguments);
  }

  /**
   * Wrapper method for obtaining parents of a given taxonomy term.
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   * @return mixed
   */
  protected function getTermParents(TermInterface $taxonomy_term) {
    return $this->entityTypeManager()->getStorage('taxonomy_term')->loadAllParents($taxonomy_term->id());
  }
}